Package: Rcpp
Title: Seamless R and C++ Integration
Version: 0.12.13
Date: 2017-09-24
Author: Dirk Eddelbuettel, Romain Francois, JJ Allaire, Kevin Ushey, Qiang Kou,
 Nathan Russell, Douglas Bates and John Chambers
Maintainer: Dirk Eddelbuettel <edd@debian.org>
Description: The 'Rcpp' package provides R functions as well as C++ classes which
 offer a seamless integration of R and C++. Many R data types and objects can be
 mapped back and forth to C++ equivalents which facilitates both writing of new
 code as well as easier integration of third-party libraries. Documentation 
 about 'Rcpp' is provided by several vignettes included in this package, via the 
 'Rcpp Gallery' site at <http://gallery.rcpp.org>, the paper by Eddelbuettel and 
 Francois (2011, JSS), the book by Eddelbuettel (2013, Springer) and the paper
 by Eddelbuettel and Balamuta (2017, PeerJ); see 'citation("Rcpp")' for details.
Depends: R (>= 3.0.0)
Imports: methods, utils
Suggests: RUnit, inline, rbenchmark, knitr, rmarkdown, pinp, pkgKitten
        (>= 0.1.2)
VignetteBuilder: knitr
URL: http://www.rcpp.org, http://dirk.eddelbuettel.com/code/rcpp.html,
        https://github.com/RcppCore/Rcpp
License: GPL (>= 2)
BugReports: https://github.com/RcppCore/Rcpp/issues
MailingList: Please send questions and comments regarding Rcpp to
        rcpp-devel@lists.r-forge.r-project.org
RoxygenNote: 6.0.1
NeedsCompilation: yes
Packaged: 2017-09-24 20:35:24 UTC; edd
Repository: CRAN
Date/Publication: 2017-09-28 08:09:26 UTC
Built: R 3.3.2; x86_64-w64-mingw32; 2017-10-05 08:58:47 UTC; windows
Archs: x64
