Package: Rcpp
Title: Seamless R and C++ Integration
Version: 1.0.14
Date: 2025-01-11
Authors@R: c(person("Dirk", "Eddelbuettel", role = c("aut", "cre"), email = "edd@debian.org",
                    comment = c(ORCID = "0000-0001-6419-907X")),
             person("Romain", "Francois", role = "aut",
                    comment = c(ORCID = "0000-0002-2444-4226")),
             person("JJ", "Allaire", role = "aut",
                    comment = c(ORCID = "0000-0003-0174-9868")),
             person("Kevin", "Ushey", role = "aut",
                    comment = c(ORCID = "0000-0003-2880-7407")),
             person("Qiang", "Kou", role = "aut",
                    comment = c(ORCID = "0000-0001-6786-5453")),
             person("Nathan", "Russell", role = "aut"),
             person("Iñaki", "Ucar", role = "aut",
                    comment = c(ORCID = "0000-0001-6403-5550")),
             person("Doug", "Bates", role = "aut",
                    comment = c(ORCID = "0000-0001-8316-9503")),
             person("John", "Chambers", role = "aut"))
Description: The 'Rcpp' package provides R functions as well as C++ classes which
 offer a seamless integration of R and C++. Many R data types and objects can be
 mapped back and forth to C++ equivalents which facilitates both writing of new
 code as well as easier integration of third-party libraries. Documentation
 about 'Rcpp' is provided by several vignettes included in this package, via the
 'Rcpp Gallery' site at <https://gallery.rcpp.org>, the paper by Eddelbuettel and
 Francois (2011, <doi:10.18637/jss.v040.i08>), the book by Eddelbuettel (2013,
 <doi:10.1007/978-1-4614-6868-4>) and the paper by Eddelbuettel and Balamuta (2018,
 <doi:10.1080/00031305.2017.1375990>); see 'citation("Rcpp")' for details.
Imports: methods, utils
Suggests: tinytest, inline, rbenchmark, pkgKitten (>= 0.1.2)
URL: https://www.rcpp.org,
        https://dirk.eddelbuettel.com/code/rcpp.html,
        https://github.com/RcppCore/Rcpp
License: GPL (>= 2)
BugReports: https://github.com/RcppCore/Rcpp/issues
MailingList: rcpp-devel@lists.r-forge.r-project.org
RoxygenNote: 6.1.1
Encoding: UTF-8
NeedsCompilation: yes
Packaged: 2025-01-11 20:21:25 UTC; edd
Author: Dirk Eddelbuettel [aut, cre] (<https://orcid.org/0000-0001-6419-907X>),
  Romain Francois [aut] (<https://orcid.org/0000-0002-2444-4226>),
  JJ Allaire [aut] (<https://orcid.org/0000-0003-0174-9868>),
  Kevin Ushey [aut] (<https://orcid.org/0000-0003-2880-7407>),
  Qiang Kou [aut] (<https://orcid.org/0000-0001-6786-5453>),
  Nathan Russell [aut],
  Iñaki Ucar [aut] (<https://orcid.org/0000-0001-6403-5550>),
  Doug Bates [aut] (<https://orcid.org/0000-0001-8316-9503>),
  John Chambers [aut]
Maintainer: Dirk Eddelbuettel <edd@debian.org>
Repository: CRAN
Date/Publication: 2025-01-12 16:10:02 UTC
Built: R 4.4.2; aarch64--netbsd; 2025-12-31 19:10:00 UTC; unix
