
History
=======

Rcpp continues and extends earlier work by Dominick Samperi which he initially
contributed directly into the RQuantLib package. Then, during 2005 and 2006,
several releases were made as CRAN packages, first under the name Rcpp and
later under the name RcppTemplate.  However, both packages were left
abandonded after November 2006.  Later, in November 2009, two brief releases
were made and abandonded immediately afterwards.

Due to the continued use of Rcpp both within RQuantLib and in other projects,
Dirk took over maintenance in November 2008 and added some extensions which
are documented in the NEWS file, the ChangeLog and the included annoucements
of the major releases.

With Romain joining development in late 2009, Rcpp development accelerated. A
richer, more powerful API has been developped to replace the previous API that 
we now call 'classic Rcpp API'. The 'classic Rcpp API' has been factored 
out of Rcpp in its own package 'RcppClassic'.


Documentation
=============

Several so-called 'vignette' pdf documents are included which provide

     o	an introduction;

     o	answers to frequently asked question;

     o	documentation about how to use Rcpp in other R packages;

     o	documentation of modules (to access C++ classes and functions);

     o  documentation of sugar (expressive C++ code for vectors and matrices);

     o	notes about extending Rcpp;

     o  results from the hundreds of unit tests.

The CRAN package RcppExamples provides additionnal documentation and
examples.


Contact
=======

Rcpp is in active and open development, and new contributors are invited to
follow the development on GitHub. The rcpp-devel project mailing list is the main
venue for discussions about Rcpp.

   -- Dirk Eddelbuettel and Romain Francois
