#!/usr/bin/env r
# -*- mode: R; tab-width: 4; -*-
#
# Copyright (C) 2014  Dirk Eddelbuettel, Romain Francois, and Kevin Ushey
#
# This file is part of Rcpp.
#
# Rcpp is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Rcpp is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Rcpp.  If not, see <http://www.gnu.org/licenses/>.

.runThisTest <- Sys.getenv("RunAllRcppTests") == "yes"

if (.runThisTest) {

    .setUp <- Rcpp:::unitTestSetup("attributes.cpp")
    test.attributes <- function() {
        checkEquals(
            comments_test(),
            "Start a C++ line comment with the characters \"//\""
        )
        checkEquals(
            parse_declaration_test(),
            "Parse function declaration"
        )
        checkEquals(
            parse_default_values_with_str_parenthesis(),
            "Parse function header with parenthis inside default string values."
        )
        checkEquals(
            parse_default_values_with_chr_parenthesis(),
            "Parse function header with parenthis inside default char values."
        )
        checkEquals(
            parse_default_values_with_chr_backslash(),
            "Parse function header with backslash inside default char values."
        )
    }

}
