\name{testRcppClass-package}
\alias{testRcppClass-package}
\alias{testRcppClass}
\docType{package}
\title{
Dummy package part of Rcpp unit testing
}
\description{
Dummy package part of Rcpp unit testing
}
\details{
\tabular{ll}{
Package: \tab testRcppClass\cr
Type: \tab Package\cr
Version: \tab 1.0\cr
Date: \tab 2010-09-06\cr
License: \tab GPL (>=2)\cr
LazyLoad: \tab yes\cr
}
}
\keyword{ package }

